//MyVector3.h
#include <math.h>
#include "MyVector3.h"

MyVector3::MyVector3() : x(0.0f), y(0.0f), z(0.0f)
{
}
//---------------------------------------------------------------------------------------------
MyVector3::MyVector3(float _x, float _y, float _z) : x(_x), y(_y), z(_z)
{
}
//---------------------------------------------------------------------------------------------
MyVector3::MyVector3(const MyVector3& v)
{
    x = v.x;
    y = v.y;
    z = v.z;	
}
//---------------------------------------------------------------------------------------------
//operator  ڵ http://www.terathon.com/code/vector4d.php  C4Engine 
const float& MyVector3::operator[] (int i) const
{
   return ((&x)[i]);
}
//---------------------------------------------------------------------------------------------
float& MyVector3::operator[] (int i)
{
   return ((&x)[i]);
}
//---------------------------------------------------------------------------------------------
// VECTOR = scalar * VECTOR
MyVector3 MyVector3::operator *(float num) const
{
	return MyVector3(x * num, y * num, z * num);
}
//---------------------------------------------------------------------------------------------
MyVector3 operator *(float num, const MyVector3& vec)
{
   return MyVector3(vec.x * num, vec.y * num, vec.z * num);
}
//---------------------------------------------------------------------------------------------
// .
float MyVector3::operator *(const MyVector3& v) const
{
    return (x * v.x + y * v.y + z * v.z);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::operator =(const MyVector3& v)
{
    x = v.x;
    y = v.y;
    z = v.z;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3 MyVector3::operator +(const MyVector3& v) const
{
    return MyVector3(x + v.x, y + v.y, z + v.z);
}
//---------------------------------------------------------------------------------------------
MyVector3 MyVector3::operator -(const MyVector3& v) const
{
    return MyVector3(x - v.x, y - v.y, z - v.z);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::operator +=(const MyVector3& v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3 MyVector3::operator /(float t) const
{
    float f = 1.0f / t;
    return (MyVector3(x * f, y * f, z * f));
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::operator /=(float t)
{
    float f = 1.0f / t;
    x *= f;
    y *= f;
    z *= f;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::Normalize()
{
	float _sqrt = sqrtf(x * x + y * y + z * z);
	if(_sqrt == 0.0f )
		return *this;
    return (*this /= _sqrt);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::Cross(const MyVector3& v1, const MyVector3& v2)
{
	x = v1.y * v2.z - v1.z * v2.y;
	y = v1.z * v2.x - v1.x * v2.z;
	z = v1.x * v2.y - v1.y * v2.x;

	return (*this);
} 
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::RotateX(float angle)
{
    float s = sinf(angle);
    float c = cosf(angle);
    
    float ny = c * y - s * z;
    float nz = c * z + s * y;
    
    y = ny;
    z = nz;
    
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::RotateY(float angle)
{
    float s = sinf(angle);
    float c = cosf(angle);
    
    float nx = c * x + s * z;
    float nz = c * z - s * x;
    
    x = nx;
    z = nz;
    
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::RotateZ(float angle)
{
    float s = sinf(angle);
    float c = cosf(angle);
    
    float nx = c * x - s * y;
    float ny = c * y + s * x;
    
    x = nx;
    y = ny;
    
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector3& MyVector3::RotateAxis(float angle, const MyVector3& axis)
{
    float s = sinf(angle);
    float c = cosf(angle);
    float k = 1.0F - c;
    
    float nx = x * (c + k * axis.x * axis.x) + y * (k * axis.x * axis.y - s * axis.z)
            + z * (k * axis.x * axis.z + s * axis.y);
    float ny = x * (k * axis.x * axis.y + s * axis.z) + y * (c + k * axis.y * axis.y)
            + z * (k * axis.y * axis.z - s * axis.x);
    float nz = x * (k * axis.x * axis.z - s * axis.y) + y * (k * axis.y * axis.z + s * axis.x)
            + z * (c + k * axis.z * axis.z);
    
    x = nx;
    y = ny;
    z = nz;
    
    return (*this);
}
//---------------------------------------------------------------------------------------------
float GetLineMagnitude(const MyVector3& src, const MyVector3& dest)
{
	MyVector3   vec;

	vec[0] = dest[0] - src[0];
	vec[1] = dest[1] - src[1];
	vec[2] = dest[2] - src[2];

	return (float)sqrt(vec * vec);
}
//---------------------------------------------------------------------------------------------
//u = ((P3  - P1) dot d)/(d dot d) = ((P3  - P1) dot d)/ (|| d||) 
//P3 Point̴
bool GetLengthLineAndPoint(const MyVector3& src, const MyVector3& dest, const MyVector3& point, float* fLength)
{
	float   u, lineLength;
	MyVector3  p;

	lineLength = GetLineMagnitude(src, dest);

	//Ʒ  u   Ȱ
	//u = (( (point.x - src.x) * (dest.x - src.x)) + ((point.y - src.y) * (dest.y - src.y)) + ((point.z - src.z) * (dest.z - src.z))) / (lineLength * lineLength);
	//u = u = (point - src) * (dest - src) / ((dest - src) * (dest - src));
	u = (point - src) * (dest - src) / (lineLength * lineLength);
    
	//    ʴ 
	if(u < 0.0f || u > 1.0f)
		return false;

	//p.x = src.x + u * (dest.x - src.x);
	//p.y = src.y + u * (dest.y - src.y);
	//p.z = src.z + u * (dest.z - src.z);
	// ּ   ´
	p = src + u*(dest - src);

	//  p point ũ⸦ Ѵ.
	*fLength = GetLineMagnitude(p, point);
	return true;
}
