//MyVertexBuffer.cpp
#include <d3d9.h>
#include <d3dx9.h>

#include "../dx_base/graphics.h"
#include "MyVertex.h"
#include "MyIndexBuffer.h"
#include "MyVertexBuffer.h"

MyVertexBuffer::MyVertexBuffer()
{
    m_pVB         = NULL; 
    m_pMyIndexBuffer = NULL; 
    m_numVertices = 0; 
    m_FVF         = 0; 
    m_vertexSize  = 0;
}
//----------------------------------------------------------------------------------------------
MyVertexBuffer::~MyVertexBuffer()
{
	Release();	
}
//----------------------------------------------------------------------------------------------
bool MyVertexBuffer::CreateBuffer( IDirect3DDevice9* pDevice, UINT numVertices, DWORD FVF, UINT vertexSize, bool bDynamic )
{
    m_numVertices = numVertices;
    m_FVF = FVF;
    m_vertexSize = vertexSize;

    // Dynamic buffers can't be in D3DPOOL_MANAGED
    D3DPOOL pool = bDynamic ? D3DPOOL_DEFAULT : D3DPOOL_MANAGED;
    DWORD usage = bDynamic ? D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC : D3DUSAGE_WRITEONLY;

    if ( FAILED( pDevice->CreateVertexBuffer( m_numVertices * m_vertexSize, usage, m_FVF, pool, &m_pVB, NULL ) ) )
    {
        DebugError( "CreateVertexBuffer failed." );
        return false;
    }

    return true;
}
//----------------------------------------------------------------------------------------------
void MyVertexBuffer::Release()
{
	SAFE_DELETE( m_pMyIndexBuffer );
    SAFE_RELEASE( m_pVB );
    m_numVertices = 0;
    m_FVF = 0;
    m_vertexSize = 0;
}
//----------------------------------------------------------------------------------------------
bool MyVertexBuffer::SetData( void *pVertices, DWORD flags )
{
    if( m_pVB == NULL )
		return false;

    char *pData;
    // Lock the vertex buffer
    if ( FAILED( m_pVB->Lock( 0, 0, (void**)&pData, flags ) ) )
    {
        DebugError( "IDirect3DVertexBuffer9::Lock failed." );
        return false;
    }

    // Copy vertices to vertex buffer
    memcpy( pData, pVertices, m_numVertices * m_vertexSize );
    
    // Unlock vertex buffer
    if( FAILED( m_pVB->Unlock() ) )
    {
        DebugError( "IDirect3DVertexBuffer9::Unlock failed." );
        return false;
    }

    return true;
}
//----------------------------------------------------------------------------------------------
void MyVertexBuffer::SetIndexBuffer( MyIndexBuffer* pIB )
{
    m_pMyIndexBuffer = pIB;
}
//----------------------------------------------------------------------------------------------
void MyVertexBuffer::Render( IDirect3DDevice9* pDevice, UINT numPrimitives, D3DPRIMITIVETYPE primitiveType )
{
    if( pDevice == NULL )
		return;       

    pDevice->SetStreamSource( 0, m_pVB, 0, m_vertexSize );
    pDevice->SetFVF( m_FVF );

    if( m_pMyIndexBuffer )
    {
        pDevice->SetIndices( m_pMyIndexBuffer->GetBuffer() );
        pDevice->DrawIndexedPrimitive( primitiveType, 0, 0, m_numVertices, 0, numPrimitives );
    }
    else
    {
        pDevice->DrawPrimitive( primitiveType, 0, numPrimitives );
    }
}