//dxeffect.cpp
#include <stdio.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <Dxerr.h>
#include <tchar.h >

#include "../dx_base/graphics.h"
#include "dxeffect.h"


CDXEffect::CDXEffect()
{
	m_pDevice = NULL;
	m_pEffect = NULL;
}
//-------------------------------------------------------------------------------
CDXEffect::~CDXEffect()
{
	Release();
}
//-------------------------------------------------------------------------------
bool CDXEffect::Load(IDirect3DDevice9* pDevice, char* szFileName)
{
	ID3DXBuffer* errorBuffer = 0;

	DWORD dwShaderFlags = 0;
#if defined( _DEBUG )
	dwShaderFlags |= D3DXSHADER_DEBUG;
	dwShaderFlags |= D3DXSHADER_NO_PRESHADER;
#endif

	HRESULT hr = D3DXCreateEffectFromFile(
		pDevice,
		szFileName,
		0,
		0,
		dwShaderFlags,
		0,
		&m_pEffect,
		&errorBuffer);

	if(errorBuffer)
	{
		char szError[1024];
		sprintf_s(szError, "%s\n", errorBuffer->GetBufferPointer());
		OutputDebugString(szError);
		SAFE_RELEASE(errorBuffer);
		_V(hr);
		return false;
	}
	_V(hr);

	if(FAILED(hr))
		return false;

	m_pDevice = pDevice;

	return true;
}
//-------------------------------------------------------------------------------
void CDXEffect::Release()
{
	if(m_pDevice)
	{
		SAFE_RELEASE(m_pEffect);
		m_pDevice = NULL;
	}
}
//-------------------------------------------------------------------------------
ID3DXEffect* CDXEffect::GetEffect()
{
	return m_pEffect;
}
//-------------------------------------------------------------------------------
bool CDXEffect::SetTechnique(const char* szTechnique)
{
	HRESULT hr;
	_V(m_pEffect->SetTechnique(szTechnique));
	if(SUCCEEDED(hr))
		hr = m_pEffect->ValidateTechnique(szTechnique);
	return SUCCEEDED(hr);
}
//-------------------------------------------------------------------------------
UINT CDXEffect::Begin()
{
	UINT uPasses;
	HRESULT hr = m_pEffect->Begin(&uPasses, 0);
	if(FAILED(hr))
		uPasses = 0;

	return uPasses;
}
//-------------------------------------------------------------------------------
bool CDXEffect::End()
{
	HRESULT hr = m_pEffect->End();
	
	return SUCCEEDED(hr);
}
//-------------------------------------------------------------------------------
bool CDXEffect::BeginPass(UINT uPass)
{
	HRESULT hr = m_pEffect->BeginPass(uPass);
	
	if(FAILED(hr))
		DXTRACE_ERR("CDXEffect::BeginPass()", hr);
	return SUCCEEDED(hr);
}
//-------------------------------------------------------------------------------
bool CDXEffect::EndPass()
{
	HRESULT hr = m_pEffect->End();
	
	return SUCCEEDED(hr);
}
//-------------------------------------------------------------------------------
bool CDXEffect::CommitChanges()
{
	HRESULT hr = m_pEffect->CommitChanges();

	return SUCCEEDED(hr);
}
//-------------------------------------------------------------------------------