//Normalize.cpp
#include "windows.h"
#include "../dx_util/MyMesh.h"
#include "TerrainChunk.h"
#include "Normalize.h"


Normalize::Normalize()
{

}
//---------------------------------------------------------------------------------
Normalize::~Normalize()
{

}
//---------------------------------------------------------------------------------
bool Normalize::Run( TerrainChunk* pChunk, float fx, float fy, float fOuterRadius, float fInnerRadius, float fStrength )
{
	float fDistSq;
	float fRadiusSq = fOuterRadius * fOuterRadius;	

	float fMin = pChunk->GetHeight( fx, fy );
	float fMax = fMin;
	
	int nVertsNum = Terrain::GetVertsNum();
	MyVector3 vec;

	// find the min and max
	for( int row = 0; row < nVertsNum; ++row )
	{
		for( int col = 0; col < nVertsNum; ++col )
		{
			//߽  Ÿ
			float distX = fx - pChunk->GetX( col );    
			float distY = fy - pChunk->GetY( row );
			fDistSq = distX * distX + distY * distY;
			if( fDistSq > fRadiusSq)
				continue;

			vec = pChunk->GetVertex( col, row );
			if( vec.z < fMin ) fMin = vec.z;
			if( vec.z > fMax ) fMax = vec.z;
		}
	}

	// avoiding divide by zero (unlikely with floats, but just in case)
	if( fMax != fMin )
	{
		// divide every height by the maximum to normalize to ( 0.0, 1.0 )
		for( int row = 0; row < nVertsNum; ++row )
		{
			for( int col = 0; col < nVertsNum; ++col )
			{
				//߽  Ÿ
				float distX = fx - pChunk->GetX( col );    
				float distY = fy - pChunk->GetY( row );
				fDistSq = distX * distX + distY * distY;
				if( fDistSq > fRadiusSq)
					continue;

				vec = pChunk->GetVertex( col, row );
				pChunk->SetHeight( col, row, fMin + ( vec.z - fMin ) / ( fMax - fMin ) * fStrength );
			}
		}
	}
	else
	{
		// if the min and max are the same, then the terrain has no height, so just clear it to 0.0.
		for( int row = 0; row < nVertsNum; ++row )
		{
			for( int col = 0; col < nVertsNum; ++col )
			{
				//߽  Ÿ
				float distX = fx - pChunk->GetX( col );    
				float distY = fy - pChunk->GetY( row );
				fDistSq = distX * distX + distY * distY;
				if( fDistSq > fRadiusSq)
					continue;
				pChunk->SetHeight( col, row, fMin );
			}
		}
	}

	return true;
}