//TerrainEdit.cpp

#include <windows.h>
#include <math.h>
#include <d3d9types.h>
#include "../dx_util/MyMesh.h"
#include "TerrainChunk.h"

#define A		( nVertsNum * row + col )
#define B		( nVertsNum * row + col + 1 )
#define C		( nVertsNum * (row + 1) + col )
#define D		( nVertsNum * (row + 1) + col + 1 )


//--------------------------------------------------------------------------------------------
TerrainChunk::TerrainChunk()
{
	m_pMesh = 0L;
}
//--------------------------------------------------------------------------------------------
TerrainChunk::~TerrainChunk()
{
	Release();
}
//--------------------------------------------------------------------------------------------
void TerrainChunk::Release()
{
	if( m_pMesh )
		delete m_pMesh;
	m_pMesh = 0L;

}
//--------------------------------------------------------------------------------------------
bool TerrainChunk::Create( float* pfHeight )
{
	if( pfHeight )
		m_pMesh = Make( pfHeight );
	else
		m_pMesh = Make();
	return true;
}
//--------------------------------------------------------------------------------------------
MyMesh* TerrainChunk::Make()
{
	MyMesh* pMesh = 0L;
	int nCellNum = Terrain::GetCellNum();
	int nVertsNum = Terrain::GetVertsNum();
	int nTriListNum = Terrain::GetTriListNum();
	int nTriNum = Terrain::GetTriNum();
	float fVertexStride = Terrain::GetVertexStride();

	pMesh = new MyMesh();
	MyVector3* pVertex = new MyVector3[ nVertsNum * nVertsNum ];
	MyVector3* pNormal = new MyVector3[ nVertsNum * nVertsNum ];
	USHORT* pIndex = new USHORT[ nCellNum * nTriListNum ];

	DWORD* pColor = NULL;
	//DWORD* pColor = new DWORD[ nVertsNum * nVertsNum ];
	
	for( int row = 0; row < nVertsNum; ++row )
	{
		for( int col = 0; col < nVertsNum; ++col )
		{
			pVertex[ row * nVertsNum + col ].x = (float)col * fVertexStride;
			pVertex[ row * nVertsNum + col ].y = (float)row * fVertexStride;
			pVertex[ row * nVertsNum + col ].z = 0.0f;

			pNormal[ row * nVertsNum + col ].x = 0.0f;
			pNormal[ row * nVertsNum + col ].y = 0.0f;
			pNormal[ row * nVertsNum + col ].z = 1.0f;

			if(pColor)
			{
				D3DCOLOR color = D3DCOLOR_XRGB(255, 0,0);
				pColor[ row * nVertsNum + col ] = color;
			}
		}
	}

	// ¦     Ȧ
	// D  CD    C   CD
	//AB, A     AB,  B
	int index = 0;
	for( int row = 0; row < nCellNum; ++row )
	{
		for( int col = 0; col < nCellNum; ++col, index += 6 )
		{
			if( 0 == (row + col) % 2 )
			{
				pIndex[ index + 0 ] = A;
				pIndex[ index + 1 ] = B;
				pIndex[ index + 2 ] = D;
				pIndex[ index + 3 ] = A;
				pIndex[ index + 4 ] = D;
				pIndex[ index + 5 ] = C;
			}
			else
			{
				pIndex[ index + 0 ] = A;
				pIndex[ index + 1 ] = B;
				pIndex[ index + 2 ] = C;
				pIndex[ index + 3 ] = B;
				pIndex[ index + 4 ] = D;
				pIndex[ index + 5 ] = C;
			}
		}
	}

	pMesh->SetNumber( nVertsNum * nVertsNum, nTriNum ); 
	pMesh->SetVertexData( pVertex );
	pMesh->SetVertexNormal( pNormal);
	pMesh->SetVertexIndex( nCellNum * nTriListNum, pIndex );
	pMesh->SetVertexColor(pColor);
	return pMesh;
}
//--------------------------------------------------------------------------------------------
MyMesh* TerrainChunk::Make( float* pfHeight )
{
	MyMesh* pMesh = 0L;
	int nCellNum = Terrain::GetCellNum();
	int nVertsNum = Terrain::GetVertsNum();
	int nTriListNum = Terrain::GetTriListNum();
	int nTriNum = Terrain::GetTriNum();
	float fVertexStride = Terrain::GetVertexStride();

	pMesh = new MyMesh();
	MyVector3* pVertex = new MyVector3[ nVertsNum * nVertsNum ];
	USHORT* pIndex = new USHORT[ nCellNum * nTriListNum ];
	
	DWORD* pColor = NULL;
	//DWORD* pColor = new DWORD[ nVertsNum * nVertsNum ];

	float* pData = pfHeight;
	for( int row = 0; row < nVertsNum; ++row )
	{
		for( int col = 0; col < nVertsNum; ++col, ++pData )
		{
			pVertex[ row * nVertsNum + col ].x = (float)col * fVertexStride;
			pVertex[ row * nVertsNum + col ].y = (float)row * fVertexStride;
			pVertex[ row * nVertsNum + col ].z = float(*pData);
			
			if(pColor)
			{
				D3DCOLOR color = D3DCOLOR_RGBA(255,0,0,0);
				pColor[ row * nVertsNum + col ] = color;
			}
		}
	}

	// ¦     Ȧ
	// D  CD    C   CD
	//AB, A     AB,  B
	int index = 0;
	for( int row = 0; row < nCellNum; ++row )
	{
		for( int col = 0; col < nCellNum; ++col, index += 6 )
		{
			if( 0 == (row + col) % 2 )
			{
				pIndex[ index + 0 ] = A;
				pIndex[ index + 1 ] = B;
				pIndex[ index + 2 ] = D;
				pIndex[ index + 3 ] = A;
				pIndex[ index + 4 ] = D;
				pIndex[ index + 5 ] = C;
			}
			else
			{
				pIndex[ index + 0 ] = A;
				pIndex[ index + 1 ] = B;
				pIndex[ index + 2 ] = C;
				pIndex[ index + 3 ] = B;
				pIndex[ index + 4 ] = D;
				pIndex[ index + 5 ] = C;
			}
		}
	}

	pMesh->SetNumber( nVertsNum * nVertsNum, nTriNum ); 
	pMesh->SetVertexData( pVertex );
	pMesh->SetVertexIndex( nCellNum * nTriListNum, pIndex );
	pMesh->SetVertexColor(pColor);
	return pMesh;	
}
//--------------------------------------------------------------------------------------------
MyMesh* TerrainChunk::GetMyMesh()
{
	return m_pMesh;
}
//--------------------------------------------------------------------------------------------
MyVector3 TerrainChunk::CalcOneNormal( int col, int row )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3 a, b, c;
	a = GetVertex( col, row );
	
	if( (col + 1) >= nVertsNum )
		b = GetVertex( col - 1, row );
	else
		b = GetVertex( col + 1, row );

	if( (row + 1) >= nVertsNum )	
		c = GetVertex( col, row - 1 );	
	else
		c = GetVertex( col, row + 1 );

	MyVector3 u, v, n;
	u = b - a;
	v = c - a;
	n.Cross( u, v);
	n.Normalize();
	return n;
}
//--------------------------------------------------------------------------------------------
MyVector3 TerrainChunk::CalcTwoNormal( int col, int row )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3 a, b, c, d;
	a = GetVertex( col, row );
	
	int colD, rowD;
	if( (col + 1) >= nVertsNum )
	{
		b = GetVertex( col - 1, row );
		colD = col -1;
	}
	else
	{
		b = GetVertex( col + 1, row );
		colD = col + 1;
	}

	if( (row + 1) >= nVertsNum )	
	{
		c = GetVertex( col, row - 1 );	
		rowD = row - 1;
	}
	else
	{
		c = GetVertex( col, row + 1 );
		rowD = row + 1;
	}

	d = GetVertex( colD, rowD );	

	MyVector3 u, v, n1;
	MyVector3 s, t, n2;

	u = b - a;
	v = c - a;
	n1.Cross( u, v);
	n1.Normalize();

	s = c - d;
	t = b - d;
	n2.Cross( s, t);
	n2.Normalize();

	return ( n1 + n2 ) * 0.5f;
}
//--------------------------------------------------------------------------------------------
MyVector3 TerrainChunk::CalcSmoothNormal( int col, int row )
{
	int nVertsNum = Terrain::GetVertsNum();
	MyVector3 vAverage = MyVector3(0.0f,0.0f,0.0f);
	float num = 0.0f;

	for (int i = row - 1; i <= row; ++i)
	{
		for (int j = col - 1; j <= col; ++j)
		{
			if (i < 0 || j < 0 || i == (nVertsNum - 1) || j == (nVertsNum -1) )
			{
				vAverage += MyVector3(0.0f, 1.0f, 0.0f);
				num += 1.0f;
			}
			else
			{
				vAverage += CalcTwoNormal( j, i );
				num += 1.0f;
			}
		}
	}
	
	return vAverage /= num;
}
//--------------------------------------------------------------------------------------------
void TerrainChunk::ComputeNormal( float fx, float fy, float fRadius )
{
	MinMax2D bound = DetermineMinMax2D( fx, fy, fRadius );

	for( int row = bound.yMin; row <= bound.yMax; ++row )
	{
		for( int col = bound.xMin; col <= bound.xMax; ++col )
		{
			MyVector3 normal = CalcTwoNormal( col, row );
			SetNormal( col, row, &normal );
		}
	}
}
//--------------------------------------------------------------------------------------------
float TerrainChunk::GetCol( float fx )
{
	return floor( fx / Terrain::Define::fVertexStride );
}
//--------------------------------------------------------------------------------------------
float TerrainChunk::GetRow( float fy )
{
	return floor( fy / Terrain::Define::fVertexStride );
}
//--------------------------------------------------------------------------------------------
float TerrainChunk::GetX( int col )
{
	return ( Terrain::Define::fVertexStride * col );
}
//--------------------------------------------------------------------------------------------
float TerrainChunk::GetY( int row )
{
	return ( Terrain::Define::fVertexStride * row );
}
//--------------------------------------------------------------------------------------------
float TerrainChunk::GetHeight( float fx, float fy )
{
    float _A, _B, _C, _D, u, v;

	int col = (int)GetCol( fx );
	int row = (int)GetRow( fy );
    _A = GetVertex( col, row ).z;
    _B = GetVertex( col + 1, row ).z;
	_C = GetVertex( col, row + 1 ).z;
	_D = GetVertex( col + 1, row + 1 ).z;

	fx /= Terrain::Define::fVertexStride;
	fy /= Terrain::Define::fVertexStride;
    u = ( fx - floor(fx) );
    v = ( fy - floor(fy) );
    float fHeight = _A*( 1.0f - u )*( 1.0f - v ) + _B*u*( 1.0f - v ) + _C*( 1.0f - u )*v + _D*u*v;
    return fHeight;
}

//--------------------------------------------------------------------------------------------
MyVector3& TerrainChunk::GetVertex(int col, int row )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3* pVerts = m_pMesh->GetVertexData();
	return pVerts[ row * nVertsNum + col ];
}
//--------------------------------------------------------------------------------------------
void TerrainChunk::SetHeight( int col, int row, float fHeight )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3* pVerts = m_pMesh->GetVertexData();
	pVerts[ row * nVertsNum + col ].z = fHeight;
}
//--------------------------------------------------------------------------------------------
const MyVector3& TerrainChunk::GetNormal( float fx, float fy )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3* normal = m_pMesh->GetVertexNormal();

	int col = (int)GetCol( fx );
	int row = (int)GetRow( fy );

	return normal[ row * nVertsNum + col ];
}
//--------------------------------------------------------------------------------------------
void TerrainChunk::SetNormal( int col, int row, MyVector3* pVector )
{
	int nVertsNum = Terrain::GetVertsNum();

	MyVector3* pNormal = m_pMesh->GetVertexNormal();
	pNormal[ row * nVertsNum + col ] = *pVector;
}
//--------------------------------------------------------------------------------------------
MinMax2D TerrainChunk::DetermineMinMax2D(  float fx, float fy, float fRadius )
{
	int	  nVertsNum = Terrain::GetVertsNum();

	MinMax2D bound;
	//  ŭ   ġ .
	bound.xMin = (int)GetCol( fx - fRadius - 1.0f);
	bound.xMax = (int)GetCol( fx + fRadius + 1.0f);
	bound.yMin = (int)GetRow( fy - fRadius - 1.0f);
	bound.yMax = (int)GetRow( fy + fRadius + 1.0f);	

	// ûũ    ʵ Ѵ.
	if( bound.xMin < 0 ) 
		bound.xMin = 0;
	if( bound.xMax >= nVertsNum ) 
		bound.xMax = nVertsNum - 1;
	if( bound.yMin < 0 ) 
		bound.yMin = 0;
	if( bound.yMax >= nVertsNum ) 
		bound.yMax = nVertsNum - 1;

	return bound;
}