//testapp.cpp
#include <windows.h>
#include "./dx_base/graphics.h"
#include "./dx_util/MyVector3.h"
#include "./dx_util/xfile.h"
#include "./dx_util/dxeffect.h"
#include "./dx_util/MyRenderer.h"
#include "./dx_util/MyMaterial.h"
#include "./terrain/TerrainChunk.h"
#include "./terrain/Raise.h"
#include "./terrain/Flatten.h"
#include "./terrain/Lower.h"
#include "./terrain/Normalize.h"
#include "./dx_util/MyBox.h"
#include "util.h"
#include "testapp.h"


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	CTestApp * pApp = new CTestApp;

	pApp->m_mainParam.hInstance = hInstance;
	pApp->m_mainParam.lpCmdLine = lpCmdLine;
	pApp->m_mainParam.nCmdShow = nCmdShow;
	
	pApp->Register(pApp, " ", 640, 480); 
	pApp->Run();
	pApp->Release();
	return 0;
}
//---------------------------------------------------------------------------------
CTestApp::CTestApp()
{
	m_pXFile = NULL;
	m_pEffect = NULL;
	m_cameraRef.wheel = 50.0f;
	m_cameraRef.fRotateY = -32.0f;
	m_pMaterial = NULL;
	m_pMyBox = NULL;
}
//---------------------------------------------------------------------------------
CTestApp::~CTestApp()
{

}
//---------------------------------------------------------------------------------
void CTestApp::InitDirectionalLight()
{
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(light) );
	light.Type = D3DLIGHT_DIRECTIONAL;

	D3DXVECTOR3 vecDir;
	vecDir = D3DXVECTOR3(0.5f, 0.5f, -0.1f);
	D3DXVec3Normalize((D3DXVECTOR3*)&light.Direction, &vecDir );

	// set directional light diffuse color
	light.Type  = D3DLIGHT_DIRECTIONAL;
	light.Diffuse = D3DXCOLOR( 0.9f, 0.9f, 0.9f, 1.0f );
	light.Ambient = D3DXCOLOR( 0.3f, 0.3f, 0.3f, 1.0f );

	pDevice->SetLight( 0, &light );
	pDevice->LightEnable( 0, TRUE );
	pDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	pDevice->SetRenderState( D3DRS_ZENABLE,  TRUE );
	//Ambient Light 
	pDevice->SetRenderState(D3DRS_AMBIENT, 0x00202020);
}
//---------------------------------------------------------------------------------
bool CTestApp::PostGraphics()
{
	CApplication::PostGraphics();
	InitDirectionalLight();

	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();
	
	m_pXFile = new CXFile;
	m_pXFile->Load(pDevice, "airplane 2.x");

	m_pRenderer = new MyRenderer;

	m_pMaterial = new MyMaterial();
	m_pMaterial->SetAmbient( 0.7f, 0.7f, 0.7f);
	m_pMaterial->SetDiffuse( 0.7f, 0.7f, 0.7f);
	m_pMaterial->SetEmissive(0.0f, 0.0f, 0.0f);
	m_pChunk = new TerrainChunk;
	m_pChunk->Create();

	//pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	int   chunkSize = ( int )Terrain::GetCellNum();

	// 
	Raise raise;
	raise.Run( m_pChunk,  m_pChunk->GetX(chunkSize / 2), m_pChunk->GetY(chunkSize / 2), 50.0f, 30.0f, 0.02f );
	Flatten flatten;
	flatten.Run( m_pChunk, m_pChunk->GetX(chunkSize / 2), m_pChunk->GetY(chunkSize / 2), 50.0f, 30.0f, 0.04f );
	raise.Run( m_pChunk, 40.0f, 40.0f, 30.0f, 10.0f, 0.01f );

	Lower lower;
	lower.Run( m_pChunk, 90.0f, 20.0f, 20.0f, 1.0f, 0.05f );
	//Normalize normalize;
	//normalize.Run( m_pChunk, m_pChunk->GetX(chunkSize / 2), m_pChunk->GetY(chunkSize / 2), 50.0f, 0.0f, 10.0f );

	//ü Ʈ ..  ū 
	m_pChunk->ComputeNormal( 0.0f, 0.0f, 1000.0f);

	m_pMyBox = new MyBox;
	MyVector3 minVec(-5, -5, 0);
	MyVector3 maxVec(5, 5, 30);
	m_pMyBox->Create(minVec, maxVec);
	m_cameraRef.wheel = 20.0f;
	return true;
}
//---------------------------------------------------------------------------------
void CTestApp::Cleanup()
{
	SAFE_DELETE(m_pEffect);
	SAFE_DELETE(m_pXFile);
	SAFE_DELETE(m_pMaterial);
	SAFE_DELETE(m_pRenderer);
	SAFE_DELETE(m_pChunk);
	SAFE_DELETE(m_pMyBox);
	CApplication::Cleanup();
}
//---------------------------------------------------------------------------------
void CTestApp::Update()
{

}
//---------------------------------------------------------------------------------
void CTestApp::Render()
{
#if  0
	D3DXMATRIX   matTranslation, matRot, matWorld;
	
	D3DXMatrixTranslation( &matTranslation, -64.0f, m_cameraRef.wheel * 2.0f, -40.0f);
	//D3DXMatrixRotationYawPitchRoll( &matRot, 0.0f, D3DXToRadian(-m_cameraRef.fRotateY), D3DXToRadian(-m_cameraRef.fRotateX));
	D3DXMatrixRotationYawPitchRoll( &matRot, 0.0f, D3DXToRadian(-m_cameraRef.fRotateY), 0.0f);
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	m_pMaterial->SetMaterial(pDevice);

	matWorld = matRot * matTranslation;
	pDevice->SetTransform( D3DTS_WORLD, &matWorld);

	MyMesh* pTerrainMesh = m_pChunk->GetMyMesh();
	m_pRenderer->Render(pDevice, *pTerrainMesh );
	//m_pXFile->Draw();

	myBox.Render(pDevice, m_pRenderer, 32.0f, 32.0f, m_pChunk->GetHeight( 32.0f, 32.0f));
	//myBox.Render(pDevice, m_pRenderer, m_cameraRef.wheel, m_cameraRef.fRotateX, );
#else
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();
    D3DXMATRIX   matProj;
	D3DXMATRIX	 matCameraView, matWorld;
	D3DXVECTOR3  cameraTarget;
    D3DXVECTOR3  cameraPos;

	// Set where camera is viewing from
	cameraPos.x =  65.0f;
	cameraPos.y = -80.0f;
	cameraPos.z =   90.0f;
	// Set what camera is looking at
	cameraTarget.x = 65.0f;
	cameraTarget.y = 90.0f;
	cameraTarget.z = 0.0f;

	// Setup Camera View
	D3DXMatrixLookAtRH( &matCameraView, 
						&cameraPos, 
						&cameraTarget, 
						&D3DXVECTOR3( 0.0f, 0.0f, 1.0f ) );	

	pDevice->SetTransform( D3DTS_VIEW, &matCameraView );


	
	MyMesh* pTerrainMesh = m_pChunk->GetMyMesh();
	m_pMaterial->SetMaterial(pDevice);

	D3DXMatrixTranslation( &matWorld, 0.0f, 0.0f, 0.0f);
	pDevice->SetTransform( D3DTS_WORLD, &matWorld);
	m_pRenderer->Render(pDevice, *pTerrainMesh );

	float loc = m_cameraRef.wheel * 4.0f;
	float fHeight = m_pChunk->GetHeight( loc, loc) + m_pMyBox->GetRadius().z;

	D3DXMATRIX rotMat;
	GetTerrainSlope(rotMat, m_pChunk, loc, loc, fHeight);
	MyVector3 pos = MyVector3(loc, loc, fHeight);
	m_pMyBox->Render(pDevice, m_pRenderer, pos, rotMat);

	// ؽĸ NULL     D3DMATERIAL9  ģ.
	pDevice->SetTexture( 0, NULL );  
#endif
}
//---------------------------------------------------------------------------------
bool CTestApp::ProcessKey(HWND hWnd, WPARAM wKey)
{
	return CApplication::ProcessKey(hWnd, wKey);
}


